function outL = graphcut_segmentation(I,inL,lambda)
% GRAPHCUT_SEGMENTATION segment image using Potts model with GMM data term. 
% terms.
%
% Synopsis:
%   outL = graphcut_segmentation(I,inL,Lambda)
%
% Input:
%  I [H x W x 3 (uint8)] RGB image of size H x W pixels.
%  inL [H x W (uint8)] Partial image annotation:
%    0 .. pixel not annotated
%    1 .. pixel assigned label 1
%    2 .. pixel assigned label 2
%  Lambda [1 x 1] Parametr of the Potts model. 
%
% Output:
%  outL [H x W] Complete image segmentation. Each pixel must be 
%    either 1 or  2.
%

if nargin < 3
    lambda = 15;
end

K = 5;

%

imH = size(I,1);
imW = size(I,2);

% segment the image by graph-cut
[E,HV] = gridgraph(imH,imW);
nE = size(E,2);

hard_idx1=find(inL==1);
hard_idx2=find(inL==2);

I=double(I);
Ir=I(:,:,1);
Ig=I(:,:,2);
Ib=I(:,:,3);

X1 = [Ir(hard_idx1) Ig(hard_idx1) Ib(hard_idx1)]';
X2 = [Ir(hard_idx2) Ig(hard_idx2) Ib(hard_idx2)]';
X = [Ir(:) Ig(:) Ib(:)]';
        
[centers,labels,objval] = k_means(X1,K);
model1 = gmm_ml(X1,labels);

[centers,labels,objval] = k_means(X2,K);
model2 = gmm_ml(X2, labels);
        
p1 = -logsumexp(gmm_logpxy(X,model1));
p2 = -logsumexp(gmm_logpxy(X,model2));

%
p1(hard_idx1) = -10000;
p2(hard_idx2) = -10000;

% segment the image by graph-cut
switch 1
    case 1
        G  = (1-eye(2,2))*lambda;
        f2 = int32( repmat(G,[1 1 nE]) );

    case 2
        sigma = 2*var(X(:));
        
        D1 = lambda*exp(-sum((I(:,1:end-1,:) - I(:,2:end,:)).^2,3)/sigma);
        D2 = lambda*exp(-sum((I(1:end-1,:,:) - I(2:end,:,:)).^2,3)/sigma)';
        
        g = [D1(:); D2(:)];

        G = zeros(2,2,nE);
        G(2,1,:) = g;
        G(1,2,:) = g;
        f2 = int32( G );
end
    
Q = [p1(:)';p2(:)'];
E  = int32( E-1 );
f1 = int32( Q );

S = emaxflow_mex(E,f1,f2);
S = reshape(S,imH,imW);   % 0..bacground-foreground, 1..object    
outL = S + 1;

return;
