function [E,HV] = gridgraph(Height,Width)
% GRIDGRAPH Creates edges of grid graph.
% 
% Synopsis:
%  [E,HV] = gridgraph(Height,Width)
%
% Description:
%  
%  Example graph [3x3]:    Function output:
%    1-4-7                   E = [ 1 2 3 4 5 6 1 4 7 2 5 8 ;
%    | | |                         4 5 6 7 8 9 2 5 8 3 6 9 ]
%    2-5-8                   HV= [ 1 1 1 1 1 1 2 2 2 2 2 2 ]
%    | | |                  
%    3-6-9                   nE = size(E,2) = 12
%                            nT = max(E(:)) = 9
%        
% Input:
%  Height [1x1] Grid height.
%  Width  [1x1] Grid width.
%
% Output:
%  E [2 x nE (uint32)] Graph edges; the i-th edge (t1,t2) is given 
%    by objects t1=E(1,i) and t2=E(2,i).
%  HV [1 x nE] Edge type; 1 - horizontal, 2-vertical.
%  

% History:
% 29-jun-07, VF
% 29-mar-07, VF
% 02-oct-06, VF, adopted (stolen :-)) from Tomas Werner's code.
 
% creates edges of a rectangular grid
E = uint32(zeros(2,(Height-1)*Width+Height*(Width-1)));
HV = uint32(zeros(1,(Height-1)*Width+Height*(Width-1)));
k = 1;
i = 1:Height;
for j = 2:Width
  E(:,k:k+Height-1) = [ i+(j-2)*Height; i+(j-1)*Height ];
  k = k + Height;
end
HV(1:k-1) = 1;
HV(k:end) = 2;

j = 1:Width;
for i = 2:Height
  E(:,k:k+Width-1) = [ i-1+(j-1)*Height; i+(j-1)*Height ];
  k = k + Width;
end

return;
% EOF